const defaultOptions = {
  suffixes: [
    'js',
    'css',
  ],
  replace: false
};

const checkOptions = (options) => {
  const assignOptions = Object.assign({}, options);
  const { suffixes, replace } = assignOptions;
  if (suffixes && !(suffixes instanceof Array)) {
    throw new Error('options.suffixes should be type of \'Array\'.');
  }
  if (replace && !(typeof replace === 'boolean')) {
    throw new Error('options.replace should be type of \'Boolean\'.');
  }
  return true;
};

const filterSuffixes = (options) => {
  if (checkOptions(options)) {
    const { suffixes: defaultSuffixes } = defaultOptions;
    const newOptions = Object.assign({}, options);
    const { suffixes, replace } = newOptions;
    if (replace) {
      if (suffixes) {
        defaultOptions.suffixes = suffixes;
      } else {
        throw new Error('you should pass options.suffixes when options.replace is \'true\'.');
      }
    } else if (suffixes) {
      const set = new Set();
      suffixes.forEach(suffix => set.add(suffix));
      defaultSuffixes.forEach(suffix => set.add(suffix));
      defaultOptions.suffixes = [...set];
    }
  }
  return defaultOptions;
};

module.exports = class StaticsVersion {
  constructor(options) {
    this.options = filterSuffixes(options);
    this.addVersion = this.addVersion.bind(this);
    this.apply = this.apply.bind(this);
  }
  apply(compiler) {
    compiler.plugin('compilation', this.addVersion);
  }
  addVersion(compilation) {
    const { suffixes } = this.options;
    compilation.plugin('html-webpack-plugin-after-html-processing', (htmlPluginData, cb) => {
      const version = +new Date();
      suffixes.forEach((suffix) => {
        const reg = new RegExp(`(\\.${suffix})`, 'g');
        htmlPluginData.html = htmlPluginData.html.replace(reg, `$1?v=${version}`);
      });
      cb(null, htmlPluginData);
    });
  }
};
